# Command line switch for verbosity
ifdef V
VERBOSE = $(V)
else
VERBOSE = 0
endif

# Quiet prefixes
ifeq ($(VERBOSE),1)
  E = :
  Q =
else
  E = echo
  Q = @
endif

#
# Locations
#

prefix = $(ARC_LOCATION)
exec_prefix = $(prefix)
bindir = $(exec_prefix)/bin
libdir = $(exec_prefix)/lib

# Defines

SWIG = swig
PERL = perl

# Implicit rules

# C++ compilation
%.o: %.cpp %.d
	@$(E) " [CXX] $<"
	$(Q)$(CXX) -c $(CPPFLAGS) $($<_CPPFLAGS) $(CXXFLAGS) $($@_CXXFLAGS) -o $@ $<

# C++ dependency generation
%.d: %.cpp
	@$(E) " [DEP] $<"
	$(Q)$(CXX) -M $(CPPFLAGS) $($<_CPPFLAGS) $< > $@.$$$$; \
        sed 's,\($*\)\.o[ :]*,\1.o $@ : ,g' < $@.$$$$ > $@; \
        rm -f $@.$$$$

$(LIBRARIES): %.so:
	@$(E) " [LD]  $@"
	$(Q)rm -f $@
	$(Q)$(CXX) -shared -o $@ $(LDFLAGS) $($*.so_OBJS) $($*.so_LIBS)

$(STATIC_LIBRARIES): %.a:
	@$(E) " [AR]  $@"
	$(Q)$(AR) r $@ $($*.a_OBJS)

$(PROGRAMS): %: %.o
	@$(E) " [LD]  $@"
	$(Q)$(CXX) -o $@ $(LDFLAGS) $($*_OBJS) $($*_LIBS)

# Python wrappers
%_wrapper.d: %.i
	@$(E) " [DEP] $<"
	$(Q)$(SWIG) -M $(CPPFLAGS) $($<_CPPFLAGS) $< > $@.$$$$; \
	sed 's,\($*\)\.o[ :]*,\1.o $@ : ,g' < $@.$$$$ > $@; \
	rm -f $@.$$$$

%_wrapper.cpp: %.i
	@$(E) " [SWG] $<"
	$(Q)$(SWIG) -I../include -Wall -c++ -python -shadow -o $@ $<
	# Hack to integrate exceptions into python
	$(PERL) -pi -e 's/class exception\(_object\):/import exceptions\nclass exception\(exceptions.Exception\):/g' $@

install: installPROGRAMS installLIBRARIES

installPROGRAMS: $(foreach bin, $(PROGRAMS), $(DESTDIR)$(bindir)/$(bin))

$(foreach bin, $(PROGRAMS), $(DESTDIR)$(bindir)/$(bin)): $(DESTDIR)$(bindir)/%: %
	@$(E) Installing $* in $(DESTDIR)$(bindir)
	$(Q)mkdir -p $(DESTDIR)$(bindir); cp $* $(DESTDIR)$(bindir)

installLIBRARIES: $(foreach lib, $(LIBRARIES), $(DESTDIR)$(libdir))

$(foreach lib, $(LIBRARIES), $(DESTDIR)$(libdir)/$(lib)): $(DESTDIR)$(libdir)/%: %
	@$(E) Installing $* in $(DESTDIR)$(libdir)
	$(Q)mkdir -p $(DESTDIR)$(libdir) ; cp $* $(DESTDIR)$(libdir)

doc: doxygen

doxygen:
	@$(E) " [DOX] "
	$(Q)doxygen doxygen.conf

.PHONY: installPROGRAMS installLIBRARIES install doc
