#ifndef FLO_MESSAGE_H
#define FLO_MESSAGE_H

#include <string>
#include <list>

#include "common.h"
#include "object.h"

/// Message sent between services for notification
class FlOMessage {
 private:
  /// Object this message notifies about
  const_pFlOObject object_;
  /// Type of message (action performed on object)
  FlOObject::action_t type_;
 public:
  FlOMessage(FlOObject::action_t t);
  FlOMessage(const FlOObject& o,FlOObject::action_t t);
  /// Convert message into form suitable for sending through channel
  void serialize(std::string& msg);
  /// Process received message. Created message have to be deleted.
  static FlOMessage* deserialize(const std::string& msg);
  /// Modify type of message
  void operator=(FlOObject::action_t t) { type_=t; };
  /// Check if message refers to object with same ID
  bool operator==(const std::string& id) const { return (object_->id() == id);};
  /// Check if message refers to same object
  bool operator==(const FlOObject& o) const { return (object_ == o); };
  /// check if message refers to same object
  bool operator==(const_pFlOObject p) const { return (object_ == p); };
  /// Get type of message
  FlOObject::action_t type(void) const { return type_; };
  /// Get object associated with message
  const_pFlOObject object(void) const { return object_; };
};

/// Registration request
class FlORegistration {
 private:
  /// Service sending request
  FlOService& service_;
 public:
  FlORegistration(FlOService& s):service_(s) { };
  FlOService& service(void) { return service_; };
  bool operator==(const FlOService& s) const { return ((&service_) == (&s)); };
};

#endif // FLO_MESSAGE_H
