package se.umu.jobsubmission.impl.dispatcher; 

import java.util.LinkedList;
import java.util.List;
import java.util.Vector;

import org.apache.axis.message.MessageElement;
import org.apache.axis.types.NormalizedString;
import org.apache.axis.types.URI;
import org.apache.log4j.Logger;
import org.ggf.schemas.jsdl._2005._11.jsdl.Application_Type;
import org.ggf.schemas.jsdl._2005._11.jsdl.CandidateHosts_Type;
import org.ggf.schemas.jsdl._2005._11.jsdl.DataStaging_Type;
import org.ggf.schemas.jsdl._2005._11.jsdl.Exact_Type;
import org.ggf.schemas.jsdl._2005._11.jsdl.JobDefinition_Type;
import org.ggf.schemas.jsdl._2005._11.jsdl.JobDescription_Type;
import org.ggf.schemas.jsdl._2005._11.jsdl.JobIdentification_Type;
import org.ggf.schemas.jsdl._2005._11.jsdl.RangeValue_Type;
import org.ggf.schemas.jsdl._2005._11.jsdl.Resources_Type;
import org.ggf.schemas.jsdl._2005._11.jsdl.SourceTarget_Type;
import org.ggf.schemas.jsdl._2005._11.jsdl_posix.Argument_Type;
import org.ggf.schemas.jsdl._2005._11.jsdl_posix.Environment_Type;
import org.ggf.schemas.jsdl._2005._11.jsdl_posix.FileName_Type;
import org.ggf.schemas.jsdl._2005._11.jsdl_posix.Limits_Type;
import org.ggf.schemas.jsdl._2005._11.jsdl_posix.POSIXApplication_Type;
import org.globus.rsl.AbstractRslNode;
import org.globus.rsl.NameOpValue;
import org.globus.rsl.RslNode;
import org.globus.rsl.Value;
import org.globus.wsrf.encoding.ObjectDeserializer;

import se.umu.jobsubmission.util.JobDescriptionTranslator;

/** 
 * This class translates JSDL job descriptions to xRSL 
 * for submission to ARC resources. 
 */

public class JSDL2GT2RSL implements JobDescriptionTranslator {

    private Logger logger = Logger.getLogger(JSDL2GT2RSL.class); 
    
    
    /*
     * arguments
     * count
     * directory
     * executable
     * environment
     * jobType
     * maxTime
     * maxWallTime
     * maxCpuTime
     * gramMyjob 	
     * stdin
     * stdout
     * stderr
     * queue
     * project
     * dryRun 		-> Not used here. 
     * maxMemory
     * minMemory
     * hostCount
     */
    
    public void translateJobIdentification(AbstractRslNode rsl, 
    		JobDescription_Type jsdl) throws Exception {
    	
    	/*
    	if(jobID != null) {
    		String jobName = jobID.getJobName(); 
    		if(jobName != null) {
    			  NameOpValue nov = new NameOpValue("jobName", NameOpValue.EQ, 
    			  		jobName); 
    			  xrsl.add(nov); 
    		}
    	} */
		// Project
    	JobIdentification_Type jobIdentification = jsdl.getJobIdentification(); 
    	if(jobIdentification != null) {
    		String projects[] = jobIdentification.getJobProject(); 
    		if(projects != null) {
    			NameOpValue nov = new NameOpValue("project", NameOpValue.EQ, 
    					projects[0]); 
    			rsl.add(nov); 
    		}
    	}
    	
    }
    
    public void translateResource(AbstractRslNode rsl, 
    		JobDescription_Type jsdl) throws Exception {
    	
       	// Resource contains cluster, count, memory, disk, etc. 
    	Resources_Type resource = jsdl.getResources(); 
    	if(resource != null) {
    		
    		/*
    		CandidateHosts_Type host = resource.getCandidateHosts(); 
    		if(host != null) {
    			String hostName = host.getHostName(0); 
    			NameOpValue nov = new NameOpValue("cluster", NameOpValue.EQ, 
    					hostName); 
    			xrsl.add(nov); 
    		}
    		*/
    		
    		RangeValue_Type totalCPUs = resource.getTotalCPUCount();
    		if(totalCPUs != null) {
    			Exact_Type exact[] = totalCPUs.getExact(); 
    			double d = exact[0].get_value(); 
    			NameOpValue nov = new NameOpValue("count", NameOpValue.EQ, 
    					new String("" + (int)d)); 
    			rsl.add(nov); 
    		}
    		
    		
    		RangeValue_Type individualCPUs = resource.getIndividualCPUCount(); 
    		if(individualCPUs != null) {
    			Exact_Type exact[] = individualCPUs.getExact(); 
    			double d = exact[0].get_value(); 
    			NameOpValue nov = new NameOpValue("hostCount", NameOpValue.EQ, 
    					new String("" + (int)d)); 
    			rsl.add(nov); 
    		}
    		
    		// N.B. use minMemory instead? 
    		RangeValue_Type memory = resource.getIndividualPhysicalMemory(); 
    		if(memory != null) {
    			Exact_Type exact[] = memory.getExact(); 
    			double d = exact[0].get_value(); 
    			NameOpValue nov = new NameOpValue("maxMemory", NameOpValue.EQ, 
    					new String("" + (int)d)); 
    			rsl.add(nov);     			
    		}
    		/*
    		RangeValue_Type diskSpace =  resource.getIndividualDiskSpace(); 
    		if(diskSpace != null) {
    			Exact_Type exact[] = diskSpace.getExact(); 
    			double d = exact[0].get_value(); 
    			NameOpValue nov = new NameOpValue("disk", NameOpValue.EQ, 
    					new String("" + (int)d)); 
    			xrsl.add(nov);     			
    		}
    		*/
    		
    		// JSDL specifies time in seconds, RSL in minutes. 
    		RangeValue_Type CPUTime =  resource.getIndividualCPUTime(); 
    		if(CPUTime != null) {
    			Exact_Type exact[] = CPUTime.getExact(); 
    			double d = exact[0].get_value(); 
    			NameOpValue nov = new NameOpValue("maxCpuTime", NameOpValue.EQ, 
    					new String("" + (((int)d)/60))); 
    			rsl.add(nov);     			
    		}
    	}
    	
    }
    
    public void translateApplication(AbstractRslNode rsl, 
    		JobDescription_Type jsdl) throws Exception {
    	
    	Application_Type application = jsdl.getApplication(); 
    	if(application != null) {
    		
    		// POSIXApplication (an extension to Application contains 
    		// executable, arguments, std{in,out,err} etc.
    		MessageElement any[] = application.get_any(); 
    		POSIXApplication_Type posix = null; 
    		posix = (POSIXApplication_Type) ObjectDeserializer.toObject(any[0], 
    				POSIXApplication_Type.class); 
    		if(posix == null) {
    			// Big trouble...
    			throw new Exception("Job translation failed. no executable specified"); 
    		}
    		else {
    			
    			FileName_Type executable = posix.getExecutable(); 
    			if(executable != null) {
    				String name = executable.get_value(); 
    				NameOpValue nov = new NameOpValue("executable", NameOpValue.EQ, 
    						name); 
    				rsl.add(nov); 
    			}
    			else 
    				throw new Exception("Job translation failed. no executable specified"); 
    			
    			
    			FileName_Type input = posix.getInput(); 
    			if(input != null) {
    				String name = input.get_value(); 
    				NameOpValue nov = new NameOpValue("stdin", NameOpValue.EQ, 
    						name); 
    				rsl.add(nov); 
    			}
    			
    			FileName_Type output = posix.getOutput(); 
    			if(output != null) {
    				String name = output.get_value(); 
    				NameOpValue nov = new NameOpValue("stdout", NameOpValue.EQ, 
    						name); 
    				rsl.add(nov); 
    			}
    			
    			FileName_Type error = posix.getError(); 
    			if(error != null) {
    				String name = error.get_value(); 
    				NameOpValue nov = new NameOpValue("stderr", NameOpValue.EQ, 
    						name); 
    				rsl.add(nov); 
    			}
    			
    			// arguments, environment. 
    			Argument_Type argument[] = posix.getArgument(); 
    			if(argument != null) {
    				int size = argument.length; 
    				String argumentValues[] = new String[size]; 
    				for(int i = 0; i < size; i++) {
    					NormalizedString name = argument[i].get_value(); 
    					argumentValues[i] = name.toString();
    				}
    				NameOpValue nov = new NameOpValue("arguments", 
    						NameOpValue.EQ, 
							argumentValues); 
    				rsl.add(nov); 
    			}
    			
    			Environment_Type environment[] = posix.getEnvironment(); 
    			if(environment != null) {
    				int size = environment.length; 
    				logger.debug("Translationg " + size + " environments"); 
    				List allEnvironments = new LinkedList(); 
    				NameOpValue nov = new NameOpValue("environment", NameOpValue.EQ); 
    				for(int i = 0; i < size; i++) {
    					List env = new LinkedList(); 
    					Value envName = new Value(environment[i].getName().toString()); 
    					Value envValue = new Value(environment[i].get_value()); 
    					env.add(envName); 
    					env.add(envValue); 
    					nov.add(env); 
    				}
    				rsl.add(nov); 
    			}
    			
    			// Don't know how GRAM gatekeeper treats this one 
    			// in relation to CPUtime...
    			Limits_Type wallTime = posix.getWallTimeLimit(); 
    			if(wallTime != null) {
    				int value =  wallTime.get_value().intValue(); 
    				NameOpValue nov = new NameOpValue("maxWallTime", 
    						NameOpValue.EQ, "" + value); 
    				rsl.add(nov); 
    			}
    			
    		}
    	} 
    	else {
    		// This means big trouble, as we have not defined an executable...
    		throw new Exception("Job description must specify \"Executable\""); 
    	}
    }
    
    /**
     * Adds a file to a file relation. rslRoot is the root node of the
     * rsl document, relationName is either 'file_stage_in' or 
     * 'file_stage_out', 
     * 
     * @param rslRoot
     * @param relationName
     * @param localFileName
     * @param remoteFileName
     */
    private void addFile(AbstractRslNode rslRoot, 
            String relationName, 
            String localFileName, 
            String remoteFileName) {

        if(localFileName == null || 
                remoteFileName == null) 
            return; 

        List newFile = new Vector(); 
        newFile.add(new Value(remoteFileName)); 
        newFile.add(new Value(localFileName)); 
        
        logger.debug("Adding: " + newFile + " to relation " + relationName); 

        NameOpValue fileRelation = rslRoot.getParam(relationName); 

        if(fileRelation == null) {
            fileRelation = new NameOpValue(relationName, NameOpValue.EQ); 
            rslRoot.add(fileRelation); 
        }
        fileRelation.add(newFile); 
    }
	
    /**
     * Translates job description (which is a jsdl JobDefinition_Type) 
     * to an AbstractRslNode, containing the GT2 style translation
     * of the job description. 
     */
    public Object translate(Object jobDesc) throws Exception {
    	
    	logger.debug("Translation description back to GT2-style RSL"); 
    	
    	JobDefinition_Type jobDefinition = (JobDefinition_Type) jobDesc; 
    	JobDescription_Type jsdl = jobDefinition.getJobDescription(); 
    	
    	AbstractRslNode rsl = new RslNode(AbstractRslNode.AND); 
    	
    	// Translate project name attribute 
    	this.translateJobIdentification(rsl, jsdl); 
    
    	// Translate attributes related to resource requirements
    	// e.g., diskspace, memory, count. 
    	this.translateResource(rsl, jsdl); 
 
    	// Translate attributes related to job configuration 
    	// e.g., executable, stdin, stdout, arguments, environment. 
    	this.translateApplication(rsl, jsdl); 
 
    	
    	/* Format for GT2 RSL file staging:     	
    	 (file_stage_in=($(GLOBUSRUN_GASS_URL)/`pwd`/append.pl append.pl)
          		(gsiftp://gridum1.cs.umu.se:2812/scratch/file1.txt in1)
          		(gsiftp://gridum2:2812/scratch/file2.txt in2)
          		(file:///Home/staff/tordsson/infile3.txt in3))
          (file_stage_out=(gsiftp://gridum2:2812/`pwd`/add.err add.err)
                 (gsiftp://gridum2:2812/`pwd`/add.out add.out)
                 (gsiftp://gridum2:2812/`pwd`/result.txt out))

    	 */
    	
    	DataStaging_Type dataStaging[] = jsdl.getDataStaging(); 
    	if(dataStaging != null) {
    		int size = dataStaging.length; 
    		for(int i = 0; i < size; i++) {
    			SourceTarget_Type source = dataStaging[i].getSource(); 
    			SourceTarget_Type target = dataStaging[i].getTarget(); 
    			String fileName = dataStaging[i].getFileName(); 
    			
    			// Is this stage-in or stage out? 
    			// target == null -> input file (stage from source to fileName)
    			// source == null -> output file (stage from target to fileName)
    			
    			if(source == null && target == null) {
    				logger.error("Incorrect specified job file"); 
    				throw new Exception("Incorrect specified job file"); 
    			}
    			
    			String name = null; 
    			// Input
    			if(source != null) {
    				name = source.getURI().toString(); 
    				this.addFile(rsl, "file_stage_in", fileName, name); 
    			}
     			// Output file
    			if(target != null) {
    				name = target.toString(); 
    				logger.debug("Name of file is: " + name); 
    				this.addFile(rsl, "file_stage_out", fileName, name); 
    			}
    		}
    		
    	}
    	
    	logger.debug("Translated RSL is:\n" + rsl.toString()); 
    	
    	return rsl; 
    }

}
