package se.umu.jobsubmission.impl.informationfinder; 

import java.util.LinkedList;
import java.util.List;

import org.apache.log4j.Logger;
import org.globus.gsi.GlobusCredential;
import org.nordugrid.job.mds.ARCMDSDiscovery;

public class ARCResourceFinder implements ResourceFinder {

    // Assume name has format ldap://grid.uio.no:2135/
    //
    static Logger logger = Logger.getLogger(ARCResourceFinder.class); 
    
    public List findResources(String indexServerURI, 
    		int timeout, 
			GlobusCredential credential) 
    	throws Exception {
    	
        try {
        	
        	// Try to separate the ARC server url from LCG ones. 
        	if(indexServerURI.matches(".*:2170.*")) {
        		logger.info("LDAP server URL suggests LCG BDII: " + indexServerURI); 
        		throw new Exception ("LDAP server URL suggests LCG BDII: " 
        				+ indexServerURI); 
        	}
        	// Ensure that we do not send queries to a GT4 container (or similar)
        	if(indexServerURI.startsWith("https")) { 
        		logger.info("URL " + indexServerURI + " suggests non-ARC information service"); 
        		throw new Exception("URL " + indexServerURI + 
        				" suggests non-ARC information service"); 
        	}

            logger.debug("Query to: " + indexServerURI); 
            ARCMDSDiscovery arcDiscovery = new ARCMDSDiscovery(indexServerURI, 
            		timeout); 
            arcDiscovery.DiscoverAll(); 

            // Only one return value allowed, make a list of lists, 
            // first list is GIISes, second list is clusters. 
            List resources = new LinkedList(); 

            List giises = new LinkedList(); 
            String giis; 
            int noGIISes = arcDiscovery.haveGIISes(); 
            for(int i = 0; i < noGIISes; i++) {
                giis = arcDiscovery.getGIIS(i); 
                giises.add(giis); 
            }

            List clusters = new LinkedList(); 
            String cluster; 
            int noClusters = arcDiscovery.haveClusters(); 
            for(int i = 0; i < noClusters; i++) {
                cluster = arcDiscovery.getCluster(i); 
                clusters.add(cluster); 
            }

            resources.add(giises); 
            resources.add(clusters); 

            logger.debug("Query to: " + indexServerURI + " done. Found " 
                    + giises.size() + " GIISes and " 
                    + clusters.size() + " clusters"); 

            return resources; 
        }
        catch (Exception e) {
            logger.warn("No resources found at " + indexServerURI); 
            throw new Exception("No resources found at: " + indexServerURI, e); 
        }
    }

}
