/*
 * Created on Feb 8, 2006
 *
 */
package se.umu.jobsubmission.impl.informationfinder;

import java.util.LinkedList;
import java.util.List;

import org.apache.axis.MessageContext;
import org.apache.axis.message.MessageElement;
import org.apache.log4j.Logger;
import org.globus.gsi.GlobusCredential;
import org.globus.wsrf.client.ServiceURL;
import org.globus.wsrf.utils.AnyHelper;
import org.oasis.wsrf.properties.QueryResourcePropertiesResponse;
import org.oasis.wsrf.properties.QueryResourceProperties_Element;
import org.oasis.wsrf.properties.QueryResourceProperties_PortType;
import org.w3c.dom.Element;

import org.globus.axis.util.Util;
import se.umu.jobsubmission.util.QueryUtils;
import se.umu.jobsubmission.util.SubmitterUtils;

/**
 * @author tordsson
 
 * This informationFinder queries an index server and retrieves
 * a list of index servers URLs. These URLs correspond to Grid resources
 * running a ManagedJobFactoryService with either a PBS, LSF och 
 * Condor batch system. 
 */
public class GT4ResourceFinder implements ResourceFinder {
	
	private static Logger logger = Logger.getLogger(GT4ResourceFinder.class); 	
	
	public List findResources(String indexServerURI, 
			int timeout, 
			GlobusCredential credential) 
	throws Exception {
		
		
		List resources = new LinkedList(); 
		try {
			
			logger.info("Index server: " + indexServerURI); 				

			String expr = "//*[local-name()='MemberServiceEPR' and "; 
			expr += "(contains(child::*/child::*/child::text(), 'PBS')"; 
			expr += " or contains(child::*/child::*/child::text(), 'LSF') "; 
			expr += " or contains(child::*/child::*/child::text(), 'Condor'))]";
			expr += "/*[local-name()='Address']"; 
			
			logger.debug("Query: " + expr); 
			
			QueryResourceProperties_Element query = QueryUtils.getQuery(expr); 
			QueryResourceProperties_PortType queryPort = QueryUtils.getQueryPort(
					indexServerURI, credential); 
			
			// Set timeout on stub: 
	        ((org.apache.axis.client.Stub)queryPort).setTimeout(timeout); 
			
			QueryResourcePropertiesResponse response = 
				queryPort.queryResourceProperties(query);				
			
			MessageElement any[] = response.get_any(); 
			List addresses = new LinkedList(); 
			if(any != null) {
				
				logger.debug("Found " + any.length + " elements"); 
				// Using DOM as there seem to exist no corresponding type for 
				// <ns1:Address xmlns:ns1="http://schemas.xmlsoap.org/ws/2004/03/addressing"
				Element elements[] = AnyHelper.toElement(any);				
				for(int i = 0; i < elements.length; i++) {
					String address = elements[i].getChildNodes().item(0).getNodeValue(); 
					addresses.add(address); 
				}
								
				// Address is now https://host:8443/wsrf/services/ManagedJobFactoryService
				// change it to https://host:8443/wsrf/services/DefaultIndexService
				for(int i = 0; i < addresses.size(); i++) {
					String address = (String) addresses.get(i); 
					ServiceURL url = new ServiceURL(address); 
					url.setServiceName("DefaultIndexService"); 
					String indexServiceURL = ServiceURL.getURL(url.getURL().getHost(), 
							 "DefaultIndexService").toString(); 					
					logger.debug("Adding URL: " + indexServiceURL); 
					resources.add(indexServiceURL); 
				}
				
			} else {
				logger.debug("Query returned no results"); 
			}

		} catch (Exception e) {
			logger.info("Error performing resource discovery at host " 
					+ indexServerURI); 
			throw new Exception("Error performing resource discovery at host " 
					+ indexServerURI); 
		}
		
		if(resources.size() == 0) {
                    logger.info("No resources found at " + indexServerURI); 
                    throw new Exception("No resources found at: " + indexServerURI); 
		}
		
		// ResourceFinder should return two lists, 
		// the first on containing found index servers (none in this case) 
		// and the others containing the found resources. 
		List results = new LinkedList(); 
		List indexServers = new LinkedList(); 
		results.add(indexServers); 
		results.add(resources); 
		
		return results; 
	}
	/*
	public static void main(String args[]) throws Exception {
		
		if(args.length != 1) {
			System.out.println("Usage: gt4-infoquery serviceURL"); 
			System.exit(0); 
		}
		List l = new GT4ResourceFinder().findResources(args[0], 0); 
		System.out.println("Found index services: "); 
		for(int i = 0; i < l.size(); i++) {
			System.out.println(l.get(i)); 
		}
	}
	*/ 
	 
}
