package org.nordugrid.job.gridftp;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Vector;

import org.apache.log4j.Logger; 
import org.globus.gsi.GlobusCredential;

public class ARCGridFTPJob {
	
	private static Logger logger = Logger.getLogger(ARCGridFTPJob.class); 

    protected ARCGridFTPJobInterface iface = null;

    protected String id = null;
    
    protected String proxylocation = null;
    
    protected GlobusCredential globusproxy = null;

    public ARCGridFTPJob(String url) throws ARCGridFTPJobException {
        iface = new ARCGridFTPJobInterface(url);
    }
    
    public ARCGridFTPJob(String url,String id) throws ARCGridFTPJobException {
        iface = new ARCGridFTPJobInterface(url);
        this.id = new String(id);       
    }

    public void addProxyLocation(String proxylocation) {
    	this.proxylocation = proxylocation;
    }

    public void addProxy(GlobusCredential globusproxy) {
    	this.globusproxy = globusproxy;
    }
	
    public void Connect() throws ARCGridFTPJobException {
        if(iface == null) {
            throw new ARCGridFTPJobException("Interface is not initialised");
        }
        if (proxylocation != null) {
        	iface.Connect(proxylocation); 
        } else if (globusproxy != null) {
        	iface.Connect(globusproxy);
        } else {
        	iface.Connect();
        }
    }

    public void Disconnect() {
        if(iface != null) iface.Disconnect();
    }

    public void Submit(String rsl) throws ARCGridFTPJobException {
        if(iface == null) {
            throw new ARCGridFTPJobException("Interface is not initialised");
        }
        if(id != null) {
            throw new ARCGridFTPJobException("Job already submitted");
        }
        try {        	
        	if(proxylocation != null) 
        		id = iface.Submit(proxylocation);
        	else if (globusproxy != null) 
        		id = iface.Submit(globusproxy);
            else 
            	id=iface.Submit();
            ByteArrayInputStream buffer = new ByteArrayInputStream(
            		rsl.getBytes("US-ASCII"));
            iface.Upload(buffer,id+"/new");
        } catch(ARCGridFTPJobException err) {
            id=null;
            throw err;
        } catch(java.io.UnsupportedEncodingException err) {
            id=null;
            throw new ARCGridFTPJobException(
            		"This implementation of Java does " + 
					"not support US-ASCII encoding");
        }
    }

    public void Submit(String rsl,Vector files,Vector names) 
    throws ARCGridFTPJobException {
        if(iface == null) {
            throw new ARCGridFTPJobException("Interface is not initialised");
        }
        if(id != null) {
            throw new ARCGridFTPJobException("Job already submitted");
        }
        if(files.size() != names.size()) {
            throw new ARCGridFTPJobException(
            		"Number of files does not match number of names");
        }
        Submit(rsl);
        // Download one by one
        for(int i = 0;i<files.size();i++) {
            String file = new String((String)(files.get(i)));
            String name = new String((String)(names.get(i)));
            // Open local file
            FileInputStream stream = null;
            try {
                stream = new FileInputStream(file);
            } catch(java.io.FileNotFoundException err) {
                throw new ARCGridFTPJobException("Input file "
                		+file+" can't be open: "+err.toString());
            }
            name=id+"/"+name;
            System.err.println("Uploading "+file+" to "+name);
            iface.Upload(stream,name);
        }
    }

    public String State() throws ARCGridFTPJobException {
        if(iface == null) {
            throw new ARCGridFTPJobException("Interface is not initialised");
        }
        if(id == null) {
            throw new ARCGridFTPJobException("Job is not yet submitted");
        }
        try {
            ByteArrayOutputStream buffer = new ByteArrayOutputStream();
            iface.Download(buffer,"info/"+id+"/status");
            String state = buffer.toString("US-ASCII"); 
            return state;
        } catch(ARCGridFTPJobException err) {
            id=null;
            throw err;
        } catch(java.io.UnsupportedEncodingException err) {
            id=null;
            throw new ARCGridFTPJobException(
            		"This implementation of Java does " + 
					"not support US-ASCII encoding");
        }
    }

    public void Cancel() throws ARCGridFTPJobException {
        if(iface == null) {
            throw new ARCGridFTPJobException("Interface is not initialised");
        }
        if(id == null) {
            throw new ARCGridFTPJobException("Job is not yet submitted");
        }
        iface.Cancel(id);
    }

    public void Clean() throws ARCGridFTPJobException {
        if(iface == null) {
            throw new ARCGridFTPJobException("Interface is not initialised");
        }
        if(id == null) {
            throw new ARCGridFTPJobException("Job is not yet submitted");
        }
        iface.Clean(id);
    }

    public Vector List() throws ARCGridFTPJobException {
        if(iface == null) {
            throw new ARCGridFTPJobException("Interface is not initialised");
        }
        if(id == null) {
            throw new ARCGridFTPJobException("Job is not yet submitted");
        }
        Vector files = new Vector();
        Vector dirs = new Vector();
        dirs.addElement(id);
        for(int i = 0;i<dirs.size();i++) {
           String dir = new String((String)(dirs.get(i)));
           iface.List(dir,dirs,files);
        }
        String start = id+"/";
        for(int i = 0;i<files.size();) {
           String file = (String)(files.get(i));
           if(file.startsWith(start)) {
              files.set(i,file.substring(start.length()));
              i++;
           } else {
              files.remove(i); 
           }
        }
        return files;
    }

    public String Id() {
        return id;
    }

    public Vector Jobs() throws ARCGridFTPJobException {
        if(iface == null) {
            throw new ARCGridFTPJobException("Interface is not initialised");
        }
        Vector files = new Vector();
        Vector dirs = new Vector();
        iface.List("",dirs,files);
        for(int i = 0;i<dirs.size();) {
           if( (((String)(dirs.get(i))).equals("new")) ||
               (((String)(dirs.get(i))).equals("/new")) ||
               (((String)(dirs.get(i))).equals("info")) ||
               (((String)(dirs.get(i))).equals("/info")) ) {
               dirs.remove(i); continue;
           }
           i++;
        }
        return dirs;
    }

    public void Get(String local_dir) throws ARCGridFTPJobException {
        if(iface == null) {
            throw new ARCGridFTPJobException("Interface is not initialised");
        }
        if(id == null) {
            throw new ARCGridFTPJobException("Job is not yet submitted");
        }
        // Get list of all files
        Vector files = List();
        // Download one by one
        for(int i = 0;i<files.size();i++) {
            String file = (String)(files.get(i));
            String dir = new String();
            int p = file.lastIndexOf('/');
            if(p > 0) dir=file.substring(0,p);
            dir=local_dir+"/"+dir;
            file=local_dir+"/"+file;
            // Create local directory
            File local_file = new File(dir);
            if(!local_file.mkdirs()) {
                // Report somehow
                continue;
            }
            // Open local file
            FileOutputStream stream = null;
            try {
                stream = new FileOutputStream(file);
            } catch(java.io.FileNotFoundException err) {
                // Report somehow
                continue;
            }
            // Transfer data
            try {
System.err.println("Downloading "+(String)(files.get(i))+" to "+file);
                iface.Download(stream,(String)(files.get(i)));
            } catch(ARCGridFTPJobException err) {
                // Report somehow
                continue;
            }
        }

    }
    public void getLog(FileOutputStream stream) throws ARCGridFTPJobException {
        if(iface == null) {
            throw new ARCGridFTPJobException("Interface is not initialised");
        }
        if(id == null) {
            throw new ARCGridFTPJobException("Job is not yet submitted");
        }
        iface.Download(stream,"info/"+id+"/errors");
    }
}

