package se.umu.wsag.client; 

import javax.xml.namespace.QName;
import javax.xml.rpc.Stub;

import org.apache.axis.message.MessageElement;
import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.log4j.Logger;
import org.ggf.ws_agreement.AgreementFactory;
import org.ggf.ws_agreement.AgreementTemplateType;
import org.ggf.ws_agreement.AgreementType;
import org.ggf.ws_agreement.ContinuingFaultType;
import org.ggf.ws_agreement.CreateAgreementInputType;
import org.ggf.ws_agreement.CreateAgreementOutputType;
import org.globus.wsrf.impl.security.authentication.Constants;
import org.oasis.wsrf.properties.GetResourcePropertyResponse;

import se.umu.wsag.impl.AgreementFactoryConstants;



/**
 * 
 * @author tordsson
 *
 * The AgreementInitiator is a simplified client 
 * for communicating with the AgreementServiceFactory. 
 * The initiator has methods for resolving the factory reference, 
 * for retrieving templates and for creating agreements. 
 * Note that this component is independent of the type of agreements
 * being created, while the ReservationClient is not.  
 */
public class AgreementInitiator {

	private static Logger logger = Logger.getLogger(AgreementInitiator.class); 
	
	
	
	/**
	 * Store a reference to the factory in the agreementinitiator. 
	 * @param factoryURL
	 * @throws Exception
	 */
	/*
	public AgreementInitiator(String factoryURL) throws Exception {
        AgreementFactoryServiceAddressingLocator locator = 
        new AgreementFactoryServiceAddressingLocator();             

        EndpointReferenceType endpoint = new EndpointReferenceType();
        endpoint.setAddress(new Address(factoryURL));
        this.factory = locator.getAgreementFactoryPort(endpoint);
        
    	// Client security descriptor file     		
        String clientDesc = "etc/wsag/client-security-config.xml"; 
        // Set descriptor on Stub             
        ((Stub)this.factory)._setProperty(
        		Constants.CLIENT_DESCRIPTOR_FILE, 
				clientDesc);  
        
        this.factoryURL = factoryURL; 
        
	}*/ 
    
	/*
    public AgreementFactory getFactory(String serviceURI) 
    	throws Exception {    	
    	 // Locate service
        AgreementFactoryServiceAddressingLocator locator = 
        new AgreementFactoryServiceAddressingLocator();             

        EndpointReferenceType endpoint = new EndpointReferenceType();
        endpoint.setAddress(new Address(serviceURI));
        AgreementFactory agreementFactory = locator.getAgreementFactoryPort(endpoint);
    	
        return agreementFactory; 
    }*/ 
    
    /**
     * Returns all agreement templates from the factory. The templates 
     * are stores as RPs in the factory.  
     * @param factory The factory to get the templates from
     * @return
     * @throws Exception If there there is any problem deserializing a template
     */           
    public AgreementTemplateType[] getTemplates(AgreementFactory factory)
    	throws Exception {    	
    	
    	logger.debug("Retrieving templates from agreement factory"); 
    	
        GetResourcePropertyResponse response = factory.getResourceProperty(
        		AgreementFactoryConstants.TEMPLATE_RP);  
        MessageElement elements[] = response.get_any();    	 
        	
        if(elements == null || elements.length == 0)
        	return null;
              
    	AgreementTemplateType template = new AgreementTemplateType(); 
    	QName templateQName = AgreementTemplateType.getTypeDesc().getXmlType();
    	
    	AgreementTemplateType templates[] = new AgreementTemplateType[elements.length];  	
    	for(int i = 0; i < elements.length; i++) {
    		AgreementTemplateType tmpTemplate = (AgreementTemplateType)
				elements[i].getValueAsType(templateQName);
       		
    		templates[i] = tmpTemplate; 
    	}
 
    	return templates;  

    }
    
  
    /**
     * Sends The agreement offer to the factory and returns the EPR
     * of the created agreement. Throws exception in case agreement 
     * could not be created. 
     * @param factory
     * @param agreementOffer
     * @return
     * @throws Exception
     */
    public EndpointReferenceType createAgreement(AgreementFactory factory, 
    		AgreementType agreementOffer) throws Exception, 
    		ContinuingFaultType {
    	
    	CreateAgreementInputType input = new CreateAgreementInputType(); 
    	input.setAgreementOffer(agreementOffer); 
        CreateAgreementOutputType result = factory.createAgreement(input);
        
        return result.getCreatedAgreementEPR();         
                
    }   

}



